﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DoubleDeckerBus
{
    public partial class FormBusConfig : Form
    {
        DrawingBus _bus = null;
        private event BusDelegate EventAddBus;

        public FormBusConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (object sender, EventArgs a) => Close();
        }

        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void DrawBus()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _bus?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _bus?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        public void AddEvent(BusDelegate ev)
        {
            if (EventAddBus == null)
            {
                EventAddBus = new BusDelegate(ev);
            }
            else
            {
                EventAddBus += ev;
            }
        }

        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _bus = new DrawingBus((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, labelColor.BackColor);
                    break;
                case "labelModifiedObject":
                    _bus = new DrawingDDB((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, labelColor.BackColor, labelExtraColor.BackColor,
                        checkBoxLadder.Checked, checkBoxStage.Checked);
                    break;
            }
            DrawBus();
        }

        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = e.Data.GetData(typeof(Color));
            if (color != null) {
                (sender as Label).BackColor = (Color)color;
            }
            if (_bus != null)
            {
                _bus.Bus.BodyColor = (Color)color;
                DrawBus();
            }
        }

        private void LabelExtraColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelExtraColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = e.Data.GetData(typeof(Color));
            if (color != null)
            {
                (sender as Label).BackColor = (Color)color;
            }
            if (_bus != null)
            {
                if (_bus is DrawingDDB bus)
                {
                    bus.SetDopColor((Color)color);
                    DrawBus();
                }
                DrawBus();
            }
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            EventAddBus?.Invoke(_bus);
            Close();
        }
    }
}
